package executors;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import commands.Command;
import commands.SimulationAction;
import commands.SimulationCommand;
import models.GameOfLife;

/**
 * Mechanism for simulating a Game of Life with some specified timer interval
 * 
 * @author wkillian
 *
 */
public class GameOfLifeSimulator {

	private static GameOfLifeSimulator singleton;

	private GameOfLife instance = null;
	private int ticksPerSecond = 0;
	private ScheduledExecutorService executor = null;

	public static GameOfLifeSimulator getInstance() {
		return singleton;
	}

	/**
	 * Initialize the Simulator executor
	 * 
	 * @param game  instance of game
	 * @param ticks ticks per second
	 */
	public static void initialize(GameOfLife game, int ticks) {
		singleton = new GameOfLifeSimulator(game, ticks);
	}

	private GameOfLifeSimulator(GameOfLife game, int ticks) {
		this.instance = game;
		this.ticksPerSecond = ticks;
		Command cmd = new SimulationCommand(this.instance, SimulationAction.INITIALIZE);
		CommandExecutor.getInstance().submit(cmd);
	}

	/**
	 * modify the ticks per second for the simulator service
	 * 
	 * @param ticks ticks per second
	 */
	public void setStepAmount(int ticks) {
		this.ticksPerSecond = ticks;
	}

	/**
	 * Begin execution. Creates a new timed dispatch executor.
	 */
	public void start() {
		Command cmd = new SimulationCommand(this.instance, SimulationAction.START);
		CommandExecutor.getInstance().submit(cmd);
		cmd = new SimulationCommand(this.instance, SimulationAction.STEP);
		this.executor = Executors.newScheduledThreadPool(1);
		this.executor.scheduleAtFixedRate(cmd::execute, 0, 1000 / this.ticksPerSecond, TimeUnit.MILLISECONDS);
	}

	/**
	 * Pause execution. Invalidates the timed dispatch.
	 */
	public void pause() {
		if (this.executor != null) {
			this.executor.shutdownNow();
		}
	}

	/**
	 * Resets the execution.
	 */
	public void reset() {
		Command cmd = new SimulationCommand(this.instance, SimulationAction.RESET);
		CommandExecutor.getInstance().submit(cmd);
	}

}
