package models;

import java.util.Random;

import mvc.Model;

public class GameOfLife extends Model {

	public static final int DEFAULT_BIRTH_LOW = 3;
	public static final int DEFAULT_BIRTH_HIGH = 6;
	public static final int DEFAULT_LIVE_LOW = 3;
	public static final int DEFAULT_LIVE_HIGH = 6;
	public static final int DEFAULT_WIDTH = 20;
	public static final int DEFAULT_HEIGHT = 15;
	public static final long DEFAULT_SEED = System.currentTimeMillis();
	
	private int birthLow;
	private int birthHigh;
	private int liveLow;
	private int liveHigh;
	private int width;
	private int height;
	private long seed;
	
	private boolean[][] matrix;
	
	public GameOfLife() {
		this.birthLow = DEFAULT_BIRTH_LOW;
		this.birthHigh = DEFAULT_BIRTH_HIGH;
		this.liveLow = DEFAULT_LIVE_LOW;
		this.liveHigh = DEFAULT_LIVE_HIGH;
		this.width = DEFAULT_WIDTH;
		this.height = DEFAULT_HEIGHT;
		this.seed = DEFAULT_SEED;
		this.matrix = null;
	}

	public int getBirthLow() {
		return birthLow;
	}

	public void setBirthLow(int birthLow) {
		if (birthLow == this.birthLow)
			return;
		this.birthLow = birthLow;
		signal("birthLow", this.birthLow);
	}

	public int getBirthHigh() {
		return birthHigh;
	}

	public void setBirthHigh(int birthHigh) {
		if (birthHigh == this.birthHigh)
			return;
		this.birthHigh = birthHigh;
		signal("birthHigh", this.birthHigh);
	}

	public int getLiveLow() {
		return liveLow;
	}

	public void setLiveLow(int liveLow) {
		if (liveLow == this.liveLow)
			return;
		this.liveLow = liveLow;
		signal("liveLow", this.liveLow);
	}

	public int getLiveHigh() {
		return liveHigh;
	}

	public void setLiveHigh(int liveHigh) {
		if (liveHigh == this.liveHigh)
			return;
		this.liveHigh = liveHigh;
		signal("liveHigh", this.liveHigh);
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		if (width == this.width)
			return;
		this.width = width;
		signal("width", this.width);
		maybeReinitialize();
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		if (height == this.height)
			return;
		this.height = height;
		signal("height", this.height);
		maybeReinitialize();
	}

	public long getSeed() {
		return seed;
	}

	public void setSeed(long seed) {
		if (seed == this.seed)
			return;
		this.seed = seed;
		signal("seed", this.seed);
	}

	/**
	 * Initialize the Game of Life
	 */
	public void initialize() {
		if (matrix == null) {
			reset();
		} else {
			throw new IllegalStateException("Cannot call initialize once allocated");
		}
	}
	
	/**
	 * resets the object IFF matrix was already initialized
	 */
	private void maybeReinitialize() {
		if (matrix != null) {
			reset();
		}
	}

	/**
	 * resets the state of the board
	 */
	public void reset() {
		this.matrix = new boolean[this.height + 2][this.width + 2];
		Random rng = new Random(this.seed);
		for (int r = 1; r <= this.height; ++r) {
			for (int c = 1; c <= this.width; ++c) {
				this.matrix[r][c] = rng.nextBoolean();
			}
		}
		signal("world", this.matrix);
	}

	/**
	 * evolves the state of the simulation by one time step
	 */
	public void stepSimulation() {
		boolean[][] old = this.matrix;
		this.matrix = new boolean[this.height + 2][this.width + 2];
		for (int r = 1; r <= this.height; ++r) {
			for (int c = 1; c <= this.width; ++c) {
				if (old[r][c]) {
					this.matrix[r][c] = inRange(this.liveLow, this.liveHigh, countNeighbors(old, r, c));
				} else {
					this.matrix[r][c] = inRange(this.birthLow, this.birthHigh, countNeighbors(old, r, c));
				}
			}
		}
		signal("world", this.matrix);
	}

	private boolean inRange(int low, int high, int value) {
		return low <= value && value <= high;
	}

	private int countNeighbors(boolean[][] m, int r, int c) {
		int count = 0;
		for (int i = r - 1; i <= r + 1; ++i) {
			for (int j = c - 1; j <= c + 1; ++j) {
				if (m[i][j]) {
					++count;
				}
			}
		}
		return count;
	}

	/**
	 * starts the simulation by resetting the matrix to its default state
	 */
	public void start() {
		if (matrix == null) {
			reset();
		}
	}
	
}
